% Construction of an implied vol surface using a function quadratic in strike.
% From David Shimko (1993) "Bounds of Probability", RISK 6, pages 33-37.

clc; clear;

% Read in Shimko's data on S&P500 Index options.
K = [325 345 360 365 375 385 390 395 400 405 410 425];
C = [66.5 46 33 27.75 20.125 13.5 9.625 7.25 5.375 3.375 1.875 0.25];
P = [0.3125 0.875 2 2.625 4.25 7.125 8.75 11 13.75 17 19.75 34];
T = 60/360;
S0 = 390.02;

% Regression to get implied dividend yield and risk-free rate.
% intercept is an estimate of S*D(t,T) and
% slope is an estimate of B(t,T).
B = regstats(C-P,K,'linear');
int   = B.beta(1);
slope = B.beta(2);

% Not quite the same values as Shimko.  Override them.
int = 387.9812;
slope = -0.991646;

% Implied risk free rate and implied dividend yield.
r = -log(-slope)/T;
r = max(r,0);
q = log(S0/int)/T;
q = max(q,0);
[r q]

% Specify parameters for bisection algorithm
a = 0.01;
b = 1;
maxiter = 1e7;
tol = 1e-7;

% Find IV for calls and puts
for i=1:length(K)
    ivC(i) = BisecBSV(S0,K(i),r,q,T,a,b,'C',C(i),tol,maxiter);
 	ivP(i) = BisecBSV(S0,K(i),r,q,T,a,b,'P',P(i),tol,maxiter);
end

% Find the total implied volatility
IVC = ivC.*sqrt(T);
IVP = ivP.*sqrt(T);

% Check that the IVs recover the market option prices
for i=1:length(K)
	C2(i) = BSPrice(S0,K(i),r,q,T,ivC(i),'C');
	P2(i) = BSPrice(S0,K(i),r,q,T,ivP(i),'P');
end
[C' C2' P' P2']

% Fit a quadratic function to the Implied Vols.
B = regstats(IVC,K,'purequadratic');
A0 = B.beta(1);
A1 = B.beta(2);
A2 = B.beta(3);
[A0 A1 A2]

% Not quite the same parameter estimates as Shimko.  Even though they will
% give the same results, override them with Shimko's estimates.
A0 =  0.89167;
A1 = -0.0038715;
A2 =  0.0000044458;

% Fitted IV for market strikes.
IVhat = A0 + A1.*K + A2.*K.^2;

% Fitted IV for a continuum of strikes.
Kc = [K(1):K(end)];
IVc = A0 + A1.*Kc + A2.*Kc.^2;

% Shimko's Figure 1 (note that his figure has 13 points,
% but it should have 12 points).
plot(K, IVC,'ro', Kc, IVc)
legend('Market Call IV', 'Fitted Call IV')
pause

% Construct the empirical (truncated) RND
dK = 0.01;
K2 = [min(K):dK:max(K)];
for i=1:length(K2);
	vol = A0 + A1*K2(i) + A2*K2(i)^2;
	Call(i) = BSPrice(S0,K2(i),r,q,T,vol,'C');
end

% First derivative w.r.t K
for i=1:length(K2)-1
	dk = K2(i+1) - K2(i);
	dc = Call(i+1) - Call(i);
	dcdk(i) = dc/dk;
end

% Second derivative w.r.t. K constructs the empirical RND
for i=1:length(dcdk)-1
	dk = K2(i+1) - K2(i);
	dc = dcdk(i+1) - dcdk(i);
	RND(i) = dc/dk;
end

% Construct the Analytical RND
for i=1:length(K2);
	vol  = A0 + A1*K2(i) + A2*K2(i)^2;
	f(i) = Shimko_PDF(S0,K2(i),r,q,T,vol,A0,A1,A2);
end

% Plot the empirical RND and the Analytical RND
plot(K2(1:end-2), RND, K2, f)
legend('Empirical RND', 'Analytical RND')
xlabel('Strike Price');
ylabel('RND');

